import glob 
import re
from ete3 import Tree
import numpy as np 
import sys, getopt
import os
import math
import random
from pathlib import Path
from os import listdir
from os.path import isfile, join 
import os.path
import copy

import time
start_time = time.time()

StatArg = 0  # 0: normal run, 1: for additive test (common genes), 2: for replacing test (random from all)
EachSide = 4 #####Default 4(8neighborhood genes)
NeighbourSpecies = 3 #2 #2
geneContent_GenePosition = 1 #1: for genes content, 2 for gene position 
Additive = 20 #Thrshold of matching percent for neigbour species for transfers that classified as additive 
Replacing = 80 #Thrshold of matching percent for neigbour species for transfers that classified as replacing 
AdditiveCompare = NeighbourSpecies # how many neighbor species to compare in order to classify into additive (default 3: all neighbor species)
ReplacingCompare = 1 # how many neighbor species to compare in order to classify into replacing (default 1: at least one neighbor species)

intGenesNumber = 2 #decide how many missing genes allowed (if the transfer gene is on the edge of the contig (missingGenes < intGenesNumber))

def main(argv):
    arg_parsing(argv)
def arg_parsing(argv):
    SIngleHGTList = ''
    speciesTreefile = ''
    genomefile = ''

    global NeighbourSpecies #-d
    global StatArg #-f
    global EachSide #-e
    global geneContent_GenePosition #c
    global Additive #-a
    global Replacing #-r
    global AdditiveCompare #-b
    global ReplacingCompare # -u
    global intGenesNumber

   
    try:
        opts, args = getopt.getopt(argv,"hg:s:n:d:f:e:c:a:r:b:u:",["gfile=","sfile=","nfile=","NeighbourSpecies=","Statistical=","EachSide=","ContentPosition=","Additive=","Replacing=","AdditiveSpecies=","RelacingSpecies="])
    except getopt.GetoptError:
        print('Error: Usage: python DART.py -g <Single-HGTs file> \n -s <rooted species tree> -n <path to the directory containing gene ordering (synteny) files> [-other \n options]')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':

            print('-g \t Single-gene HGTs file. This is a required parameter. \n')
            print('-s \t Rooted species tree. This is a required parameter. \n')
            print('-n \t Path to the directory containing gene ordering (synteny) files. This is a required \n \t parameter. \n')
            print('-d \t Number of closest phylogenetic neighbors of the recipient genome to consider during \n \t classification. Defaults to 3. \n ')
            print('-e \t Number of neighborhood genes to consider on each side of the transferred gene. \n \t Defaults to 4. \n')
            #print('-c 1|2 \t Method of comparison. 1: for gene content, 2: for gene position \n \t Defaults to 1.')
            print('-a \t Genic neighborhood conservation threshold for additive transfers. Must be between 0 and \n \t 99. Defaults to 20; i.e., HGTs for which the genic neighborhood conservation is no \n \t more than 20% are classified as additive. \n')
            print('-r \t Genic neighborhood conservation threshold for replacing transfers. Must be between 0 and \n \t 99. Defaults to 80; i.e., HGTs for which the genic neighborhood conservation is at \n \t least 80% are classified as replacing. \n')
            print('-b \t Phylogenetic neighborhood conservation threshold for additive transfers. Must be \n \t between 1 and specified number of closest phylogenetic neighbors. Defaults to 3. \n')
            print('-u \t Phylogenetic neighborhood conservation threshold for replacing transfers. Must be \n \t between 1 and specified number of closest phylogenetic neighbors. Defaults to 1. \n')
            print('-f 1|2 \t Perform randomization analysis to estimate false positive classification rate. 1: for \n \t estimating false positive rate for additive classification, 2: for estimating false positive \n \t rate for replacing classification. \n')            
            print('-h \t Output this help message. \n')

            sys.exit()
        elif opt in ("-g", "--gfile"):
            SIngleHGTList = arg
        elif opt in ("-s", "--sfile"):
            speciesTreefile = arg
        elif opt in ("-n", "--nfile"):
            genomefile = arg
        elif opt in ("-e", "--EachSide"):
            EachSide = arg
        elif opt in ("-d", "--NeighbourSpecies"):
            NeighbourSpecies = arg
        elif opt in ("-c", "--ContentPosition"):
            geneContent_GenePosition = arg
        elif opt in ("-f", "--Statistical"):
            StatArg = arg
        elif opt in ("-a", "--Additive"):
            Additive = arg
        elif opt in ("-r", "--Replacing"):
            Replacing = arg
        elif opt in ("-b", "--AdditiveSpecies"):
            AdditiveCompare = arg
        elif opt in ("-u", "--RelacingSpecies"):
            ReplacingCompare = arg   

    NoEx = ''   
    executeFlag = ''
    if SIngleHGTList != None and SIngleHGTList != '' and speciesTreefile != None and speciesTreefile != '' and genomefile != None and genomefile != '':
        if (os.path.exists(SIngleHGTList) and os.path.exists(speciesTreefile) and os.path.exists(genomefile)):
            executeFlag = 'true'
        else:
            print('Error: Usage: python DART-single.py -g <Single-HGTs file> -s <rooted species tree> -n <path to the directory containing gene ordering (synteny) files> [-other options] \n Please make sure the files and folder provided have correct names.')
            NoEx = 'f'
    else:
        print('Error: Usage: python DART-single.py -g <Single-HGTs file> -s <rooted species tree> -n <path to the directory containing gene ordering (synteny) files> [-other options] \n Please make sure the files and folder provided have correct names.')
        NoEx = 'f'
    #AdditiveCompare
    if (int(AdditiveCompare) > int(NeighbourSpecies) ):
        print('Error: Number of neighbor species to compare for additive transfers must be less than or equal to the number of neighbor species')
        NoEx = 'f'
    else:
        executeFlag = 'true'
    #ReplacingCompare
    if (int(ReplacingCompare) > int(NeighbourSpecies) ):
        print('Error: Number of neighbor species to compare for replacing transfers must be less than or equal to the number of neighbor species')
        NoEx = 'f'
    else:
        executeFlag = 'true'
    ## neighbor species 
    if (int(NeighbourSpecies) <= 0 ):
        print('Error: Number of neighbor species must be 1 or more.')
        NoEx = 'f'
    else:
        executeFlag = 'true'
    # Neighborhood genes
    if (EachSide != None and EachSide !='' and EachSide > 0):# and spaceArg.isdigit() == True):#type(spaceArg) == int):
        executeFlag = 'true'
    else:
        print('Error: Number of neighborhood genes must be integer and greater than or equal to 1.')
        NoEx = 'f'

    #Additive
    if (int(Additive) >= 0 and int(Additive) <=100):
        executeFlag = 'true'
    else:
        print("Error: Additive threshold must be greater than or equal to 0 and less than or equal to 100")
        NoEx = 'f'
    
    #Replacing
    if (int(Replacing) >= 0 and int(Replacing) <=100):
        executeFlag = 'true'
    else:
        print("Error: Replacing threshold must be greater than or equal to 0 and less than or equal to 100")
        NoEx = 'f'
    
    #geneContent_GenePosition 
    if (int(geneContent_GenePosition) == 1 or int(geneContent_GenePosition) ==2):
        executeFlag = 'true'
    else:
        print("Error: Method of comparison must be either 1 for gene content or 2 for gene position.")
        NoEx = 'f'
    # Statisitical
    if (int(StatArg) == 0 or int(StatArg) ==1 or int(StatArg) == 2 or int(StatArg) == 3 or int(StatArg) == 4 ):
        executeFlag = 'true'
    else:
        print("Error: Parameter must be 0 for normal run, 1 for randomization analysis for additive transfers. 2 for randomization analysis for additive transfers. Defaults to 0.")
        NoEx = 'f'
    
    if (NoEx == 'f'):
        sys.exit()
    else:
        Transfer(SIngleHGTList, speciesTreefile, genomefile)

class Transfer:
    def __init__(self, path,speciesTree, genomesPath):
        self.genes = set()
        self.nodes = []
        self.leafNodes = []
        self.dict_nodes = {} 
        self.AdditiveCount = 0
        self.ReplacingCount = 0
        self.unassigned = 0
        self.confirmed_additive = 0
        self.confirmed_replacing = 0
        self.HMGT100Conservation = 0
        self.nodes_matrix = np.empty
        self.HGT_matrix = np.empty #matrix will have single trnsfers (node_matrix - HMGTs_matrix)
        self.nodes_counter = 1 #starts from 1 because we need the first element to be zero, that will help when building the matrix
        self.species_tree = speciesTree
        self.genomes = genomesPath+'/'
        self.SingleHGTFilePath = path
        self.AllGenomes_dict = {} # dictionary that has all species and all thier genefamily_geneName
        self.AllGenomes_lists_dict = {}
        self.AllGenomes_contig_dict = {}
        self.initiate_species_tree()
    
    #Read all the species in the species tree in postorder and put them in a list, I used list instead of dictionry because list has index that help us in the matrix 
    def initiate_species_tree(self):
        self.nodes.append('0') #The first element is zero, that will help when building the matrix
        #read the species tree and solve the problem if there are an empty lines before the species tree
        my_file = Path(self.species_tree)
        #InternalFlag = 'f'
        #Nname = ''
        if my_file.is_file():
            with open(self.species_tree, "r") as f:
                try:
                    for line in f:
                            if not line.strip():#if there are an empty lines
                                continue
                            else:
                                species_tree = line #Read the species tree in a single line 
                                self.species_tree = Tree(species_tree, format=1)
                                #preorder: 1)Visit the root, 2) Traverse the left subtree , 
                                # 3) Traverse the right subtree. postorder: 1) Traverse the 
                                # left subtree , 2) Traverse the right subtree, 3) Visit the 
                                # root levelorder (default): every node on a level before is 
                                # visited going to a lower level
                                for node in self.species_tree.traverse("postorder"): 
                                    if node.is_leaf():
                                        self.leafNodes.append(node.name)
                                        self.nodes.append(node.name)
                                break
                        
                except IOError:
                    print("Can't open the files", self.species_tree)
                except Exception as ex:
                    print(ex)
            
            self.nodes = list(filter(None, self.nodes))
            self.nodes_counter = len(self.nodes)
            #####################################################################
            #### Fill dictionary with all genomes 
            genefam_geneNameList = []
            speciesList = []
            path = self.genomes
            if os.path.exists(path): #check if genome directory found
                onlyfiles = [f for f in listdir(path) if isfile(join(path, f))]
                if not onlyfiles:
                    print("Genomes folder is empty")
                else:
                    for file in onlyfiles:
                        get_file_name = os.path.basename(file)
                        sp_name = get_file_name[0:get_file_name.find('.')] #remove .txt.sy
                        #print(file)
                        
                        genefam_geneNameList , genefam_geneNameList_contig = self.getGenomeList(path+file) # get list of geneFamily_geneNames
                        #append 0's at the first and the end of the list
                        for x in range (0,int(EachSide)):
                            speciesList.append('0')
                        for v in genefam_geneNameList:
                            speciesList.append(v)
                        for x in range (0,int(EachSide)):
                            speciesList.append('0')
                        key = sp_name
                        if key in self.AllGenomes_dict:
                            self.AllGenomes_dict[key].extend(speciesList)
                        else:
                            self.AllGenomes_dict[key] = []
                            self.AllGenomes_dict[key].extend(speciesList) 
                        ### contig list
                        if key in self.AllGenomes_contig_dict:
                            self.AllGenomes_contig_dict[key].extend(genefam_geneNameList_contig)
                        else:
                            self.AllGenomes_contig_dict[key] = []
                            self.AllGenomes_contig_dict[key].extend(genefam_geneNameList_contig) 
                        genefam_geneNameList[:] = [] #empty the list
                        genefam_geneNameList_contig[:]=[]
                        speciesList[:] = []
                        ####################################################################  
                        ##################################
                        genefam_geneNameList_ofLists = self.getGenomeList_ListOfLists(path+file) # get list of lists geneFamily_geneNames
                        #append 0's at the first and the end of the list
                        
                        key = sp_name
                        if key in self.AllGenomes_lists_dict:
                            self.AllGenomes_lists_dict[key].extend(genefam_geneNameList_ofLists)
                        else:
                            self.AllGenomes_lists_dict[key] = []
                            self.AllGenomes_lists_dict[key].extend(genefam_geneNameList_ofLists) 
            #for key in self.AllGenomes_dict:
            #    print(key, self.AllGenomes_dict[key])

            self.update_matrix()#Build the matrix from the list of the species where the rows represent the Donors and the the columns represent the recipients
        else:
            print("Species tree file ", self.species_tree, "is not exist. ")    

        
    
    # For Normal run
    def update_matrix(self):
        #Build the single HGTmatrix from the file
        HGT_matrixX = np.empty
        HGT_matrixX = np.empty((self.nodes_counter, self.nodes_counter), dtype = np.object)
        HGT_matrixX[0] = self.nodes #Fill the first row in the matrix with the nodes
        for x in range(1, len(self.nodes)):#create an empty set in each cell in the matrix
            for y in range(1, len(self.nodes)):
                HGT_matrixX[x][y] = set()        
        n = 0
        for i in self.nodes:
            HGT_matrixX[n][0] = i
            n = n + 1  
        #Fill Single HGT matrix
        # file Donor>recipient: gnfam_DgnNane_recGeneName
        HGT_dict = {}
        my_file = Path(self.SingleHGTFilePath)
        if my_file.is_file():
            with open(self.SingleHGTFilePath, "r") as f:
                try:
                    for line in f:
                            if not line.strip():#if there are an empty lines
                                continue
                            else:
                                List1 = line.split(':')
                                D_R = List1[0] # donor>recipient
                                transfersList = List1[1].split(',')
                                for xx in transfersList:
                                    if D_R in HGT_dict:
                                        HGT_dict[D_R].append(xx.strip()) #genefamily_DonorGeneName_RecipientGeneName
                                    else:
                                        HGT_dict[D_R] = []
                                        HGT_dict[D_R].append(xx.strip()) 

                        
                except IOError:
                    print("Can't open the files", self.species_tree)
                except Exception as ex:
                    print(ex)
           
        #Fill the single HGT matrix
        for i in range(1, self.nodes_counter):
            for j in range(1, self.nodes_counter):
                for keyF in HGT_dict:
                    donor = keyF.split('>')[0]
                    recipient = keyF.split('>')[1]
                    if (donor == self.nodes[i] and recipient == self.nodes[j]):
                        HGT_matrixX[i][j] = set(HGT_dict[keyF])
                                    
        if int(StatArg) == 0:
            self.print_Matrix(HGT_matrixX)
        elif int(StatArg) == 1 or int(StatArg) == 2:
            self.print_Matrix_Statisitcs(HGT_matrixX) #for additive and replacing statistical analysis neighbor species method
        else:
            self.print_Matrix_Statisitcs_D_R(HGT_matrixX) #for additive and replacing statistical analysis Donor-recipint method
        timeXX = time.time() - start_time
        print("Time needed to finish the run : "+str(timeXX))

    def getGenomeList_ListOfLists(self, file_name):
        cont = '_contig_'
        flag = 0
        get_file_name = os.path.basename(file_name)
        get_path = os.path.dirname(file_name)
        sp_name = get_file_name[0:get_file_name.find('.')] #remove .txt.sy
        genomeList = []
        contig = ''
        listOfLists = []
        tempContig = ''
        TemplistOfLists = []
        if (file_name):
            with open(file_name, 'r') as f:
                for line in f:
                    if not line.strip(): #skip empty lines
                        continue
                    else:
                        if cont in line:#check if contig in file, otherwise get the string between first and second :
                            contig = line.split('_contig_')[-1].split(':')[0].strip()
                            conPrefix = '_contig_'+contig
                        else:
                            l = line.split(':')
                            contig = l[1]
                            conPrefix = contig
                        if sp_name in line:
                            species = line.split(sp_name)  #each gene info 
                        tempContig = contig
                        geneCounter = 0
                        flag = 0

                        TemplistOfLists = []
                        genomeList = [] #empty the list
                        for s_species in species:
                            
                            if flag == 0: #to solve the problem of overcounting as the first one is empty
                                flag = 1
                            else:
                                #print("Species Info: ", s_species)
                                geneFamily = s_species.split(':')[-1].strip()
                                geneName = s_species.split(':')[1].strip()
                                key = geneFamily+'>'+geneName
                                genomeList.append(key)       
                        
                        for x in range (0,int(EachSide)):
                            TemplistOfLists.append('0')
                        for v in genomeList:
                            TemplistOfLists.append(v)
                        for x in range (0,int(EachSide)):
                            TemplistOfLists.append('0')
                        #genomeList[:] = [] #empty the list
                        listOfLists.append(TemplistOfLists)
                        #TemplistOfLists[:] = []
                    
        else:
            print("File not exist") 
        return  listOfLists
        
    def getGenomeList(self, file_name):
        cont = '_contig_'
        flag = 0
        geneCounter = 0
        get_file_name = os.path.basename(file_name)
        get_path = os.path.dirname(file_name)
        sp_name = get_file_name[0:get_file_name.find('.')] #remove .txt.sy
        genomeList = []
        genomeList_contig = []
      
        if (file_name):
            with open(file_name, 'r') as f:
                for line in f:
                    if not line.strip(): #skip empty lines
                        continue
                    else:
                        if cont in line:#check if contig in file, otherwise get the string between first and second :
                            contig = line.split('_contig_')[-1].split(':')[0].strip()
                            conPrefix = '_contig_'+contig
                        else:
                            l = line.split(':')
                            contig = l[1]
                            conPrefix = contig
                        if sp_name in line:
                            species = line.split(sp_name)  #each gene info 
                        
                        geneCounter = 0
                        flag = 0
                        for s_species in species:
                            if flag == 0: #to solve the problem of overcounting as the first one is empty
                                flag = 1
                            else:
                                #print("Species Info: ", s_species)
                                geneFamily = s_species.split(':')[-1].strip()
                                geneName = s_species.split(':')[1].strip()
                                geneCounter = geneCounter + 1
                                key = geneFamily+'>'+geneName
                                key2 = geneFamily+'>'+geneName +'*'+contig+'>'+str(geneCounter)
                                
                                genomeList.append(key)  
                                genomeList_contig.append(key2)     
        else:
            print("File not exist") 
        return  genomeList , genomeList_contig

    
    def get_neigbour_matchingPercent(self, NeigbourSpeciesX, gnf, RecipintName, TransferGene_R):
        SpeciesGfam1 = self.get_neigbour_gf_gn(NeigbourSpeciesX, gnf) # 'F' if gene not there
        neig_gf_gn1 = 'N'
        if len(SpeciesGfam1) == 0:
            neig_gf_gn1 = 'F' 
        ### get the information for the first neighbor
        speTransGene = ''
        D_B1, D_A1, R_B1, R_A1 = [], [], [], []
        Per_geneContent_neibourS_1 = -1
        if neig_gf_gn1 != 'F':
            for Spec_geneFam in SpeciesGfam1:
                Neig1FamilyList, Match_species1, D_B1x, D_A1x, R_B1x, R_A1x = self.printAround(RecipintName, NeigbourSpeciesX, str(TransferGene_R), str(Spec_geneFam))
                ### if there are more than gene family(transfer gene) in the neighbor species then select the 
                #one that have a largest matching percent
                if Match_species1 > Per_geneContent_neibourS_1:
                    Per_geneContent_neibourS_1 = Match_species1
                    D_B1, D_A1, R_B1, R_A1 = D_B1x, D_A1x, R_B1x, R_A1x
                    speTransGene = Spec_geneFam

        ## Check how many neighbor genes missing 
        S_notSpecified = 0
        SzerosCount = R_B1.count('0') + R_A1.count('0')
        #How many genes to check if they are missing
        #CheckGenes = ((int(EachSide)*2)*int(Additive))/100
        #intGenesNumber = int(CheckGenes)
        if SzerosCount > intGenesNumber:
            S_notSpecified = 1
            
            #print(NeigbourSpeciesX,R_B1,speTransGene, R_A1 )
        return Per_geneContent_neibourS_1 , R_B1,speTransGene, R_A1, S_notSpecified                 
    
    def get_continuousGenes(self, genFam, genName, speciesX):
        contig, gnNum = '',''
        genome_X = self.AllGenomes_contig_dict[speciesX]
        for itm in genome_X: #gnFa>geName*contig>genNum
            gnfam_gnName = itm.split('*')[0]
            if gnfam_gnName == genFam+'>'+genName :
                contig_num = itm.split('*')[1]
                contig = contig_num.split('>')[0]
                gnNum = contig_num.split('>')[1]
                break
        return contig, gnNum
            


    def print_Matrix(self, matrixP): ### print single HGT
        key_list = []
        pairs = 0 
        HGT = 0 
        Final_Additive, Final_Additive_DR = 0, 0
        Final_Replacing, Final_Replacing_DR = 0, 0
        Final_notClass, Final_notClass_DR = 0, 0
        final_class_add, final_class_rep, final_class_not = 0, 0, 0
        for i in range(1, self.nodes_counter):
            for j in range(1, self.nodes_counter):
                
                if len(matrixP[i][j]) != 0:
                    
                    gSet = matrixP[i][j]
                    print('Donor: '+self.nodes[i]+', Recipient: '+self.nodes[j])
                    pairs = pairs + 1

                    for g in gSet:
                        HGT = HGT + 1
                        key_list = g.split('>')
                        gfam = key_list[0] # gene family
                        gNamD = key_list[1] #Donor gene name
                        gNamR = key_list[2] # reipient gene name
        
                        contigD, geneNumD = self.get_continuousGenes(gfam,gNamD, self.nodes[i]) #contig, gene number
                        contigR, geneNumR = self.get_continuousGenes(gfam,gNamR, self.nodes[j]) #contig, gene number
                        print('Contig: '+contigD+', Gene Number: '+geneNumD+', Gene Name: '+gNamD+', Gene Family: '+gfam+' -----> Contig: '+contigR+', Gene Number: '+geneNumR+', Gene Name: '+gNamR+', Gene Family: '+gfam)

                        TransferGene_D = gfam+'>'+gNamD
                        TransferGene_R = gfam+'>'+gNamR
                        
                        ####################################################################
                        RecipientGeneFamilyList, MatchingPercent, D_B, D_A, R_B, R_A = self.printAround(self.nodes[i], self.nodes[j], str(TransferGene_D), str(TransferGene_R))
                        
                        print(' ')
                        print('Donor: ',D_B,TransferGene_D,D_A)
                        print('Recipient: ',R_B,TransferGene_R,R_A)
                        
                        D_notSpecified, R_notSpecified = 0, 0
                        DzerosCount = D_B.count('0') + D_A.count('0')
                        RzerosCount = R_B.count('0') + R_A.count('0')
                        if DzerosCount > intGenesNumber:
                            D_notSpecified = 1
                        if RzerosCount > intGenesNumber:
                            R_notSpecified = 1
                        ### Neigbour species 
                        #### check the transfer genes if they are in neigbour genes if yes then check the neigbour
                        #### genes around that gense and compare with recipient neigbour genes
                        RecipintName = self.nodes[j]
                        # get the neihbor species
                        NeigbourList = self.get_neigbourSpeciesList(self.nodes[i], self.nodes[j]) #get the neighbor species of the recipient 
                        #Random gene from species 1 and species 2
                        Per_geneContent_neibourS_dict = {}
                        
                        #get the neighbor species matching percent, compare to the recipient
                        z_Species = 0 #how many species have all the neighborhood genes
                        for neigSpe in NeigbourList: 
                            # species matching percent with the recipient, gene that match transfer gene(-1 if not found), list of genes befor the gene, and list after
                            matPercent_Species , R_B1,speTransGene, R_A1, S_notSpecified = self.get_neigbour_matchingPercent(neigSpe, gfam,RecipintName, TransferGene_R)
                            key = neigSpe
                            if S_notSpecified == 0:
                                z_Species = z_Species + 1
                            if key in Per_geneContent_neibourS_dict:
                                Per_geneContent_neibourS_dict[key].append(matPercent_Species)
                                Per_geneContent_neibourS_dict[key].append(R_B1)
                                Per_geneContent_neibourS_dict[key].append(speTransGene)
                                Per_geneContent_neibourS_dict[key].append(R_A1)
                                Per_geneContent_neibourS_dict[key].append(S_notSpecified)

                            else:
                                Per_geneContent_neibourS_dict[key] = []
                                Per_geneContent_neibourS_dict[key].append(matPercent_Species)  
                                Per_geneContent_neibourS_dict[key].append(R_B1)
                                Per_geneContent_neibourS_dict[key].append(speTransGene)
                                Per_geneContent_neibourS_dict[key].append(R_A1)
                                Per_geneContent_neibourS_dict[key].append(S_notSpecified)

                        #Print the list of genes in the neighbor species     
                        for perSpecies in Per_geneContent_neibourS_dict:
                            if Per_geneContent_neibourS_dict[perSpecies][2] != '':
                                print(perSpecies,Per_geneContent_neibourS_dict[perSpecies][1],Per_geneContent_neibourS_dict[perSpecies][2],Per_geneContent_neibourS_dict[perSpecies][3])
                            
                           
                        print('')
                        print('The neighborhood conservation percent between the donor '+self.nodes[i]+' and recipient '+self.nodes[j]+': '+str(MatchingPercent))
                        # if some of the neighbor genes are not there (transfer gene on the edge of contig)
                        #if D_notSpecified == 1 or R_notSpecified  == 1:
                        #    print('No enough information for the donor-recipient classification method.')
                        #print the matching percent for the neighbor species
                        for perSpecies in Per_geneContent_neibourS_dict:
                            if Per_geneContent_neibourS_dict[perSpecies][0] == -1:
                                print('Transfer gene not in neighbor species '+perSpecies)
                            else:
                                print('The neighborhood conservation percent between the neighbor species '+perSpecies +' and recipient: '+str(Per_geneContent_neibourS_dict[perSpecies][0]))
                        
                        #Final classifiction neighbor - species method
                        neig_classification = ''  
                        addC, repC = 0 , 0
                        for neig_per in Per_geneContent_neibourS_dict:
                            if Per_geneContent_neibourS_dict[neig_per][0] < int(Additive) and Per_geneContent_neibourS_dict[neig_per][4] == 0:
                                addC = addC + 1
                            elif  Per_geneContent_neibourS_dict[neig_per][0] > int(Replacing):
                                repC = repC +  1
                            
                        ### if neighbor species matches
                        if addC >= int(AdditiveCompare) and R_notSpecified ==0 :
                            neig_classification = 'Additive'
                            Final_Additive = Final_Additive + 1
                        elif addC < int(AdditiveCompare) and addC > 0 and addC == z_Species and R_notSpecified == 0:
                            neig_classification = 'Additive'
                            Final_Additive = Final_Additive + 1
                        elif repC >= int(ReplacingCompare) and R_notSpecified == 0:
                            neig_classification = 'Replacing'
                            Final_Replacing = Final_Replacing + 1
                        else:
                            if R_notSpecified == 1:
                                neig_classification = 'Ambiguous, no enough information.'
                            else:
                                neig_classification = 'Ambiguous'
                            Final_notClass = Final_notClass + 1
                        
                        #Final classifiction donor-recipient method
                        D_R_classification = '' 
                        if MatchingPercent < int(Additive) and D_notSpecified != 1 and R_notSpecified != 1:
                            D_R_classification = 'Additive'
                            Final_Additive_DR = Final_Additive_DR + 1
                        elif  MatchingPercent > int(Replacing) and D_notSpecified != 1 and R_notSpecified != 1:
                            D_R_classification = 'Replacing'
                            Final_Replacing_DR = Final_Replacing_DR + 1
                        else: 
                            if R_notSpecified == 1 or D_notSpecified == 1:
                                D_R_classification = 'Ambiguous, no enough information'
                            else:
                                D_R_classification = 'Ambiguous'
                            #D_R_classification = 'Ambiguous'
                            Final_notClass_DR = Final_notClass_DR + 1
                        ##############################################
                        print(' ')
                        
                        if neig_classification == 'Ambiguous':
                            neig_classification == 'Ambiguous'
                        
                        if int(geneContent_GenePosition) == 1: #Gene content
                            print('Final classification, where < '+str(Additive)+'% for additive, and > '+str(Replacing)+'% for replacing:')
                        else: # Gene position
                            print('Final classification, where < '+str(Additive)+'% for additive, and > '+str(Replacing)+'% for replacing:')
                        print(neig_classification)
                        #print('Neighbor-species method: '+neig_classification)
                        #print('Donor-recipient method: '+D_R_classification)
                        
                        final_classification = ''
                        if neig_classification ==  D_R_classification:
                            final_classification = neig_classification
                            if final_classification == 'Additive':
                                final_class_add = final_class_add +1 
                            elif final_classification == 'Replacing':
                                final_class_rep = final_class_rep + 1
                            else:
                                final_class_not = final_class_not + 1
                        elif neig_classification == 'Additive' and  D_R_classification=='Replacing':
                            
                            final_classification = 'Conflict classification'
                        elif neig_classification == 'Replacing' and  D_R_classification=='Additive':
                            final_classification = 'Conflict classification'
                        else:
                            final_classification = 'Ambiguous'
                        print("______________________________________________________________________________________________________")
                    print("______________________________________________________________________________________________________")
        
        print('**********************************************************')
        print('Summary results:')
        print('Total number of donor-recipient pairs: '+str(pairs))
        print('Total number of HGTs: '+str(HGT))
        print('')
        #print('Neighbor-species method:')
        print('Additive transfers : '+str(Final_Additive))
        print('Replacing transfers: '+str(Final_Replacing))
        print('Ambiguous transfers : '+str(Final_notClass))
        print('')
        #print('Donor-recipient method:')
        #print('Additive transfers : '+str(Final_Additive_DR))
        #print('Replacing transfers: '+str(Final_Replacing_DR))
        #print('Ambiguous transfers : '+str(Final_notClass_DR))
        '''
        print('')
        print('Using both methods:')
        print('Additive transfers : '+str(final_class_add))
        print('Replacing transfers: '+str(final_class_rep))
        print('Ambiguous transfers : '+str(final_class_not))
        '''
        print('**********************************************************')

    ##Statisitcs
    def print_Matrix_Statisitcs(self, matrixP): ### print single HGT
        pairs = 0 
        HGT = 0 
        Final_Additive, Final_Additive_DR = 0, 0
        Final_Replacing, Final_Replacing_DR = 0, 0
        Final_notClass, Final_notClass_DR = 0, 0
        for i in range(1, self.nodes_counter):
            for j in range(1, self.nodes_counter):
                
                if len(matrixP[i][j]) != 0:
                    gSet = matrixP[i][j] # set of genFam_DonrGeneName_RecipGeneName
                    NUmberOfTransfers = len(gSet)
                    print('Donor: '+self.nodes[i]+', Recipient: '+self.nodes[j])
                    pairs = pairs + 1
                    
                    RecipintName = self.nodes[j]
                    # get the neihbor species
                    NeigbourList = self.get_neigbourSpeciesList(self.nodes[i], RecipintName) #get the neighbor species of the recipient 
                    
                    #Recipient all genes #get list of all genes  geneFamily+'>'+geneName
                    GF_GN_list_R = self.AllGenomes_dict[RecipintName]  
                    
                    # copy to new list
                    genefam_geneNameList_recip = GF_GN_list_R[:]
                    
                    #Remove the first and last items that added as zeros 
                    for q in range (0, int(EachSide)): #First elements 
                        genefam_geneNameList_recip.pop(0)
                    genefam_geneNameList_recip = genefam_geneNameList_recip[:-int(EachSide)] #Last elements


                    randomRecipientList = []
                    dict_allNeighSpecies = {}
                    genefam_geneNameList_Neig = []
                        
                    if int(StatArg) == 1: # Additive test (common gene)
                        list_lists = []
                        l_R = []
                        Final_common_genes = []
                        #save only gene family 
                        for xv in genefam_geneNameList_recip:
                            l_R.append(xv.split('>')[0])
                        list_lists.append(l_R)
                        #get all the genes for the neighbor species in fill in a directory
                        for neigSpecies in NeigbourList:
                            l_S = []
                            #AllGenomes_dict
                            genefam_geneNameList_Neig_neigh = self.AllGenomes_dict[neigSpecies]
                            #Copy to another list to avoid changing items by reference
                            genefam_geneNameList_Neig = genefam_geneNameList_Neig_neigh[:]
                            #Remove the first and last items that added as zeros 
                            for q in range (0, int(EachSide)): #First elements 
                                genefam_geneNameList_Neig.pop(0)
                            genefam_geneNameList_Neig = genefam_geneNameList_Neig[:-int(EachSide)] #Last elements

                            ### get only gene family
                            for xvS in genefam_geneNameList_Neig:
                                l_S.append(xvS.split('>')[0])
                            #add it to the list of lists
                            list_lists.append(l_S)
                            # get list of common genes 
                            common_set = set(list_lists[0])
                            for s in list_lists[1:]:
                                common_set.intersection_update(s)
                            
                        #convert set to list 
                        common_list = list(common_set)
                        ## pick random common genes that matches the orginal HGT
                        if len(common_list) >= NUmberOfTransfers:
                            for d in range(0, NUmberOfTransfers):
                                randItem = random.choice(common_list)
                                Final_common_genes.append(randItem)
                                common_list.remove(randItem)
                        elif len(common_list) > 0 and len(common_list) < NUmberOfTransfers:
                            Final_common_genes = common_list[:]
                        else:
                            print('No common genes found in the recipient and the neighbor species.')
                        #set the number of NUmberOfTransfers to match the found 
                        NUmberOfTransfers = len(Final_common_genes)
                        #get the geneFamily_geneName for the common genes from the recipient
                        if len(Final_common_genes) > 0:
                            for comn in Final_common_genes:
                                for gnf_gnNa_R in genefam_geneNameList_recip:
                                    if comn == gnf_gnNa_R.split('>')[0]:
                                        randomRecipientList.append(gnf_gnNa_R)
                                        break
                            #get common genes that match the orginal number of HGTs
                            for neigSpecies_S in NeigbourList:
                                key = neigSpecies_S
                                genefam_geneNameList_Neig_S = self.AllGenomes_dict[neigSpecies_S] 
                                for comnS in Final_common_genes:
                                    for gnf_gnNa_S in genefam_geneNameList_Neig_S:
                                        if comnS == gnf_gnNa_S.split('>')[0]:
                                            if key in dict_allNeighSpecies:
                                                dict_allNeighSpecies[key].append(gnf_gnNa_S)
                                            else:
                                                dict_allNeighSpecies[key] = []
                                                dict_allNeighSpecies[key].append(gnf_gnNa_S)
                                            break
                             
                    elif int(StatArg) == 2: # replacing test (random gene)
                            #Pick a random genes from the recipient 
                        for d in range(0, NUmberOfTransfers):
                            randItemR = random.choice(genefam_geneNameList_recip)
                            randomRecipientList.append(randItemR)
                            genefam_geneNameList_recip.remove(randItemR)

                        #Pick a random gnfam_genName from the the neighbor species
                        for neigSpecies in NeigbourList:
                            genefam_geneNameList_Neig_neigh = self.AllGenomes_dict[neigSpecies] 
                            #Copy to another list to avoid changing items by reference
                            genefam_geneNameList_Neig = genefam_geneNameList_Neig_neigh[:]
                            #Remove the first and last items that added as zeros 
                            for q in range (0, int(EachSide)): #First elements 
                                genefam_geneNameList_Neig.pop(0)
                            genefam_geneNameList_Neig = genefam_geneNameList_Neig[:-int(EachSide)] #Last elements

                            for d in range(0, NUmberOfTransfers):
                                randItem = random.choice(genefam_geneNameList_Neig)
                                key = neigSpecies
                                if key in dict_allNeighSpecies:
                                    dict_allNeighSpecies[key].append(randItem)
                                else:
                                    dict_allNeighSpecies[key] = []
                                    dict_allNeighSpecies[key].append(randItem)
                                genefam_geneNameList_Neig.remove(randItem) #remove selected item from the main list
                            genefam_geneNameList_Neig[:]=[]
                        
                    #Loop through the randomly selected genes that matches the orginal number of HGTs
                    for g in range(0,NUmberOfTransfers):

                        HGT = HGT + 1
                        TransferGene_R = randomRecipientList[g] #gnf+'>'+gnR
                        
                        RecipientGeneFamilyList, MatchingPercent, D_B, D_A, R_B, R_A = self.printAround(self.nodes[j], self.nodes[j], str(TransferGene_R), str(TransferGene_R))
                        #print('after',genefam_geneNameList_recip)
                        #print(TransferGene_R)
                        print(' ')
                        print('Recipient: ',R_B,TransferGene_R,R_A)
                        
                        ### Neigbour species 
                        Per_geneContent_neibourS_dict = {}
                        
                        #get the neighbor species matching percent, compare to the recipient
                        for neigSpe in NeigbourList: 
                            #Neighbor species transfer gene
                            TransferGene_S = dict_allNeighSpecies[neigSpe][g]
                            #Check if the randomly select gene from the recipient matches the randomly selected genes from the neighbor species
                            RecipientGeneFamilyList, matPercent_Species, R_B, R_A, S_B, S_A = self.printAround(self.nodes[j], neigSpe, str(TransferGene_R), str(TransferGene_S))
                            #print(neigSpe ,S_B,TransferGene_S,S_A)
                            #matPercent_Species , R_B1,speTransGene, R_A1 = self.get_neigbour_matchingPercent(neigSpe, gnf,RecipintName, TransferGene_R)
                            key = neigSpe
                            if key in Per_geneContent_neibourS_dict:
                                Per_geneContent_neibourS_dict[key].append(matPercent_Species)
                                Per_geneContent_neibourS_dict[key].append(S_B)
                                Per_geneContent_neibourS_dict[key].append(TransferGene_S)
                                Per_geneContent_neibourS_dict[key].append(S_A)

                            else:
                                Per_geneContent_neibourS_dict[key] = []
                                Per_geneContent_neibourS_dict[key].append(matPercent_Species)
                                Per_geneContent_neibourS_dict[key].append(S_B)
                                Per_geneContent_neibourS_dict[key].append(TransferGene_S)
                                Per_geneContent_neibourS_dict[key].append(S_A)

                        #Print the list of genes in the neighbor species     
                        for perSpecies in Per_geneContent_neibourS_dict:
                            print(perSpecies,Per_geneContent_neibourS_dict[perSpecies][1],Per_geneContent_neibourS_dict[perSpecies][2],Per_geneContent_neibourS_dict[perSpecies][3])
                            
                           
                        print('')
                        #print the matching percent for the neighbor species
                        for perSpecies in Per_geneContent_neibourS_dict:
                            print('The neighborhood conservation percent between the neighbor species '+perSpecies +' and recipient: '+str(Per_geneContent_neibourS_dict[perSpecies][0]))
                        
                        #Final classifiction neighbor - species method
                        neig_classification = ''  
                        addC, repC = 0 , 0
                        for neig_per in Per_geneContent_neibourS_dict:
                            if Per_geneContent_neibourS_dict[neig_per][0] < int(Additive):
                                addC = addC + 1
                            elif  Per_geneContent_neibourS_dict[neig_per][0] > int(Replacing):
                                repC = repC +  1
                            
                        ### if neighbor species matches
                        if addC >= int(AdditiveCompare):
                            neig_classification = 'Additive'
                            Final_Additive = Final_Additive + 1
                        elif repC >= int(ReplacingCompare):
                            neig_classification = 'Replacing'
                            Final_Replacing = Final_Replacing + 1
                        else:
                            neig_classification = 'Ambiguous'
                            Final_notClass = Final_notClass + 1

                        
                        #Final classifiction donor-recipient method
                        D_R_classification = '' 
                        if MatchingPercent < int(Additive):
                            D_R_classification = 'Additive'
                            Final_Additive_DR = Final_Additive_DR + 1
                        elif  MatchingPercent > int(Replacing):
                            D_R_classification = 'Replacing'
                            Final_Replacing_DR = Final_Replacing_DR + 1
                        else: 
                            D_R_classification = 'Ambiguous'
                            Final_notClass_DR = Final_notClass_DR + 1
                        ##############################################
                        print(' ')
                        
                        if neig_classification == 'Ambiguous':
                            neig_classification == 'Ambiguous'
                        if int(geneContent_GenePosition) == 1: #Gene content
                            print('Final classification, where < '+str(Additive)+'% for additive, and > '+str(Replacing)+'% for replacing:')
                        else: # Gene position
                            print('Final classification, where < '+str(Additive)+'% for additive, and > '+str(Replacing)+'% for replacing:')
                        print(neig_classification)
                        #print('Neighbor-species method: '+neig_classification)
                        #print('Donor-recipient method: '+D_R_classification)
                        
                        print("______________________________________________________________________________________________________")
                    print("______________________________________________________________________________________________________")
        
        print('**********************************************************')
        print('Summary results:')
        print('Total number of donor-recipient pairs: '+str(pairs))
        print('Total number of HGTs: '+str(HGT))
        print('')
        #print('Neighbor-species method:')
        print('Additive transfers : '+str(Final_Additive))
        print('Replacing transfers: '+str(Final_Replacing))
        print('Ambiguous transfers : '+str(Final_notClass))
        '''
        print('')
        print('Donor-recipient method:')
        print('Additive transfers : '+str(Final_Additive_DR))
        print('Replacing transfers: '+str(Final_Replacing_DR))
        print('Ambiguous transfers : '+str(Final_notClass_DR))
        '''
        print('**********************************************************')

    ##Statisitcs - Donor-recipient method method
    def print_Matrix_Statisitcs_D_R(self, matrixP): ### print single HGT
        pairs = 0 
        HGT = 0 
        Final_Additive, Final_Additive_DR = 0, 0
        Final_Replacing, Final_Replacing_DR = 0, 0
        Final_notClass, Final_notClass_DR = 0, 0
        for i in range(1, self.nodes_counter):
            for j in range(1, self.nodes_counter):
                
                if len(matrixP[i][j]) != 0:
                    gSet = matrixP[i][j] # set of genFam_DonrGeneName_RecipGeneName
                    NUmberOfTransfers = len(gSet)
                    #print('Donor: '+self.nodes[i]+', Recipient: '+self.nodes[j])
                    pairs = pairs + 1
                    
                    DonorName = self.nodes[i]
                    RecipintName = self.nodes[j]
                    # get the neihbor species
                    #NeigbourList = self.get_neigbourSpeciesList(self.nodes[i], RecipintName) #get the neighbor species of the recipient 
                    #Recipient all genes #get list of all genes  geneFamily+'>'+geneName
                    GF_GN_list_R = self.AllGenomes_dict[RecipintName]  
                    GF_GN_list_D = self.AllGenomes_dict[DonorName]  

                    # copy to new list
                    genefam_geneNameList_recip = GF_GN_list_R[:]
                    genefam_geneNameList_donor = GF_GN_list_D[:]
                    
                    #Remove the first and last items that added as zeros 
                    for q in range (0, int(EachSide)): #First elements 
                        genefam_geneNameList_recip.pop(0)
                        genefam_geneNameList_donor.pop(0)
                    genefam_geneNameList_recip = genefam_geneNameList_recip[:-int(EachSide)] #Last elements
                    genefam_geneNameList_donor = genefam_geneNameList_donor[:-int(EachSide)] #Last elements

                    randomRecipientList, randomDonorList = [],[]
                    
                    dict_allNeighSpecies = {}
                    genefam_geneNameList_Neig = []
                        
                    if int(StatArg) == 3: # Additive test (common gene)
                        list_lists = []
                        l_R, l_D = [], []
                        Final_common_genes = []
                        #save only gene family 
                        for xv in genefam_geneNameList_recip:
                            l_R.append(xv.split('>')[0])
                        list_lists.append(l_R)

                        for xv in genefam_geneNameList_donor:
                            l_D.append(xv.split('>')[0])
                        list_lists.append(l_D)
                        
                        # get list of common genes 
                        common_set = set(list_lists[0])
                        common_set.intersection_update(set(list_lists[1]))
                        
                        #convert set to list 
                        common_list = list(common_set)
                        ## pick random common genes that matches the orginal HGT
                        if len(common_list) >= NUmberOfTransfers:
                            for d in range(0, NUmberOfTransfers):
                                randItem = random.choice(common_list)
                                Final_common_genes.append(randItem)
                                common_list.remove(randItem)
                        elif len(common_list) > 0 and len(common_list) < NUmberOfTransfers:
                            Final_common_genes = common_list[:]
                        else:
                            print('No common genes found in the recipient and the neighbor species.')
                        #set the number of NUmberOfTransfers to match the found 
                        NUmberOfTransfers = len(Final_common_genes)
                        #get the geneFamily_geneName for the common genes from the recipient
                        if len(Final_common_genes) > 0:
                            for comn in Final_common_genes:
                                for gnf_gnNa_R in genefam_geneNameList_recip:
                                    if comn == gnf_gnNa_R.split('>')[0]:
                                        randomRecipientList.append(gnf_gnNa_R)
                                        break
                            #randomDonorList
                                for gnf_gnNa_D in genefam_geneNameList_donor:
                                    if comn == gnf_gnNa_D.split('>')[0]:
                                        randomDonorList.append(gnf_gnNa_D)
                                        break

                        
                    elif int(StatArg) == 4: # replacing test (random gene)
                        #Pick a random genes from the recipient 
                        for d in range(0, NUmberOfTransfers):
                            randItemR = random.choice(genefam_geneNameList_recip)
                            randomRecipientList.append(randItemR)
                            genefam_geneNameList_recip.remove(randItemR)
                            #randomDonorList
                            randItemD = random.choice(genefam_geneNameList_donor)
                            randomDonorList.append(randItemD)
                            genefam_geneNameList_donor.remove(randItemD)
                            
                        
                    #Loop through the randomly selected genes that matches the orginal number of HGTs
                    for g in range(0,NUmberOfTransfers):

                        HGT = HGT + 1
                        TransferGene_D = randomDonorList[g] #gnf+'>'+gnR
                        TransferGene_R = randomRecipientList[g] #gnf+'>'+gnR
                        
                        RecipientGeneFamilyList, MatchingPercent, D_B, D_A, R_B, R_A = self.printAround(self.nodes[i], self.nodes[j], str(TransferGene_D), str(TransferGene_R))
                        
                        print(' ')
                        print('Donor: ',D_B,TransferGene_D,D_A)
                        print('Recipient: ',R_B,TransferGene_R,R_A)
                        D_notSpecified, R_notSpecified = 0, 0
                        DzerosCount = D_B.count('0') + D_A.count('0')
                        RzerosCount = R_B.count('0') + R_A.count('0')
                        if DzerosCount > intGenesNumber:
                            D_notSpecified = 1
                        if RzerosCount > intGenesNumber:
                            R_notSpecified = 1
                        print('')
                        print('The neighborhood conservation percent between the donor '+self.nodes[i]+' and recipient '+self.nodes[j]+': '+str(MatchingPercent))
                        # if some of the neighbor genes are not there (transfer gene on the edge of contig)
                        #if D_notSpecified == 1 or R_notSpecified  == 1:
                        #    print('No enough information for the donor-recipient classification method.')
                        D_R_classification = '' 
                        if MatchingPercent < int(Additive) and D_notSpecified != 1 and R_notSpecified != 1:
                            D_R_classification = 'Additive'
                            Final_Additive_DR = Final_Additive_DR + 1
                        elif  MatchingPercent > int(Replacing) and D_notSpecified != 1 and R_notSpecified != 1:
                            D_R_classification = 'Replacing'
                            Final_Replacing_DR = Final_Replacing_DR + 1
                        else: 
                            if R_notSpecified == 1 or D_notSpecified == 1:
                                D_R_classification = 'Ambiguous, no enough information'
                            else:
                                D_R_classification = 'Ambiguous'
                            #D_R_classification = 'Ambiguous'
                            Final_notClass_DR = Final_notClass_DR + 1
                        ##############################################
                        ##############################################
                        print(' ')
                        
                        if int(geneContent_GenePosition) == 1: #Gene content
                            print('Final classification, where < '+str(Additive)+'% for additive, and > '+str(Replacing)+'% for replacing:')
                        else: # Gene position
                            print('Final classification, where < '+str(Additive)+'% for additive, and > '+str(Replacing)+'% for replacing:')
        
                        #print('Neighbor-species method: '+neig_classification)
                        print('Donor-recipient method: '+D_R_classification)
                        
                        print("______________________________________________________________________________________________________")
                    print("______________________________________________________________________________________________________")
        
        print('**********************************************************')
        print('Summary results:')
        print('Total number of donor-recipient pairs: '+str(pairs))
        print('Total number of HGTs: '+str(HGT))
        
        print('')
        print('Donor-recipient method:')
        print('Additive transfers : '+str(Final_Additive_DR))
        print('Replacing transfers: '+str(Final_Replacing_DR))
        print('Ambiguous transfers : '+str(Final_notClass_DR))
        
        print('**********************************************************')
    
    #Return genefamily_geneName from the neigbour species if gene family exist, otherwise return -1 
    def get_neigbour_gf_gn(self, neigbrSpecies, gfam):
        NeighborGene = 'F'
        SpeciesGfam = []
        negSpeciesList = self.AllGenomes_dict[neigbrSpecies] 
        for dd in negSpeciesList:
            if '>' in dd:
                if dd.split('>')[0].strip() == gfam:
                    SpeciesGfam.append(dd)
                    #NeighborGene = dd
                    #break 
            
        return SpeciesGfam


    #Get the neigbour species
    def get_neigbourSpeciesList(self, donorNode, mainNode):
        neighbourList = []
        neigbourNodes = []
        PhloDist_numNodes = 0
        for nodesx in self.nodes:
            if (nodesx != mainNode and nodesx !='0'):
                #get all the nodes and the distance to the main orginal node 
                nNum = self.species_tree.get_distance(mainNode,nodesx, topology_only=True)  #Number of nodes
                dist = self.species_tree.get_distance(mainNode,nodesx) #phylo distance
                # Calculate distance between two descendants of current node
                if ( dist != (nNum+1)): # if phylo distance in the species tree, then use it, otherwise use number of nodes
                    distx = dist
                    PhloDist_numNodes = 1 #Phlogenetic distance 
                else:
                    ### Number of nodes
                    distx = nNum
                    PhloDist_numNodes = 2
                
                txt = str(distx)+'>'+nodesx
                neigbourNodes.append(txt)
        if PhloDist_numNodes == 1: #Phlogenetic distance 
            #phylogenetic distance
            neigbourNodes = sorted(neigbourNodes, key=lambda x: float(x.split('>')[0]))
        elif PhloDist_numNodes == 2:
            #Number of nodes
            neigbourNodes = sorted(neigbourNodes, key=lambda x: int(x.split('.')[0])) 
        
        #for p in range (0, int(NeighbourSpecies)):
        #    neighbourList.append(neigbourNodes[p].split('>')[1])
        for p in neigbourNodes:
            if (p.split('>')[1] != donorNode): # and len(neighbourList) != NeighbourSpecies):
                neighbourList.append(p.split('>')[1])
                if len(neighbourList) == int(NeighbourSpecies):
                    break
        
        return neighbourList
    
    
    #retuen a list with surrounding gene families
    def getAroundList(self, ListDR, idx, isHMGT):
        DRlist = []
        startIdx = idx - int(EachSide)
        lastIdx = idx + int(EachSide)
        #Case of single transfer, then look for 4 genes before and 4 genes after
        if (isHMGT == 'Single'):
            for x in range (startIdx, lastIdx+1):
                DRlist.append(ListDR[x]) #add the items from the list (should be the gene families around)
        #Case of HMGTs and we need to look for the only 4 before the first gene family
        elif (isHMGT == 'HMGTsF'):
            for x in range (startIdx, idx):#idx+1):
                DRlist.append(ListDR[x]) #add the items from the list (should be the gene families around)
        #Case of HMGTs, last gene family, and we need to compare gene families after this one
        elif (isHMGT == 'HMGTsL'):
            for x in range (idx+1, lastIdx+1): #(idx, lastIdx+1):
                DRlist.append(ListDR[x]) #add the items from the list (should be the gene families around)
        return DRlist
    
    
    #get the exact position comaparison _ cross
    def getPercentExactPosition(self, list1, list2, EachSideVal):
        largest_idx = EachSideVal - 1
        matching = 0
        for i in range(0,EachSideVal):
            if list1[i] == list2[largest_idx]:
                if list1[i] != '0':
                    matching = matching + 1
            largest_idx = largest_idx - 1 
        percentX = (matching * 100) / float(EachSideVal * 2)
        return percentX
        
    #get the exact position comaparison _ same
    def getPercentExact_same_Position(self, list1, list2, EachSideVal):
        matching = 0
        for i in range(0,EachSideVal):
            if list1[i] == list2[i]:
                if list1[i] != '0':
                    matching = matching + 1
        percentX = (matching * 100) / float(EachSideVal * 2)
        return percentX
    
    def printAround(self, donor, recipient, TransferGene_D, TransferGene_R):
        donorList = []
        recipientList = []
        #get the donor and the recipients whole 
        #donorList  =   self.AllGenomes_dict[donor] 
        #recipientList  = self.AllGenomes_dict[recipient] 
        
        D_list  =  self.AllGenomes_lists_dict[donor] #self.getGenomeList(self.genomes+donor+extArg)
        R_list  = self.AllGenomes_lists_dict[recipient] #self.getGenomeList(self.genomes+recipient+extArg) 
        #print(donorList)
        donorList = []
        recipientList = []
        for d_rr in D_list:
            if TransferGene_D in d_rr:
                donorList  = d_rr
        for r_rr in R_list:
            if TransferGene_R in r_rr:
                recipientList  = r_rr

        if (TransferGene_D != '0,'): #avoid missing genes 
            DTransIndexF = donorList.index(TransferGene_D) #(gfam+'>'+gNamD)
            DlistBefore = self.getAroundList(donorList, DTransIndexF, 'HMGTsF')

        if (TransferGene_D != '0,'): #avoid missing genes 
            DTransIndexL = donorList.index(TransferGene_D) #(gfam+'>'+gNamD)
            DlistAfter = self.getAroundList(donorList, DTransIndexL, 'HMGTsL')
            
        if (TransferGene_R != '0,'): #avoid missing genes 
            RTransIndexF = recipientList.index(TransferGene_R) #(gfam+'>'+gNamD)
            RlistBefore = self.getAroundList(recipientList, RTransIndexF, 'HMGTsF')

        if (TransferGene_R != '0,'): #avoid missing genes 
            RTransIndexL = recipientList.index(TransferGene_R) #(gfam+'>'+gNamD)
            RlistAfter = self.getAroundList(recipientList, RTransIndexL, 'HMGTsL')
        D_B = [] #gene families before transfer gene family for the donor
        D_A = [] #gene families after transfer gene family for the donor
        R_B = [] #gene families before transfer gene family for the recipient
        R_A = [] #gene families after transfer gene family for the recipient
        D_All = []
        R_All = []
        
        #leave only gene family, remove the gene name
        for w in DlistBefore:
            D_B.append(w.split('>')[0])
        for w in DlistAfter:
            D_A.append(w.split('>')[0])
        for w in RlistBefore:
            R_B.append(w.split('>')[0])
        for w in RlistAfter:
            R_A.append(w.split('>')[0]) 
        ########Get the functions for the donor and recipient gene families and check the percent
        D_All = D_B[:]
        D_All.extend(D_A) # all gene families in donor
        R_All = R_B[:]
        R_All.extend(R_A)# all gene families in recipient
        sameOrderPercent_total, crossOrderPercent_total = 0, 0
        DCountZero = D_All.count('0')
        RCountZero = R_All.count('0')
        maxZero = max(DCountZero, RCountZero)
 
        
        ######Do the List comparision###########
        if int(geneContent_GenePosition) == 1: #Allow the user to choose gene content or gene position 
            sameOrderPercent1 = self.getPercentX(D_B, R_B, int(EachSide), maxZero)
            sameOrderPercent2 = self.getPercentX(D_A, R_A, int(EachSide), maxZero)
            sameOrderPercent_total = sameOrderPercent1/2 + sameOrderPercent2/2
            # (D_B and R_A) + (D_A + R_B)
            crossOrderPercent1 = self.getPercentX(D_B, R_A, int(EachSide), maxZero)
            crossOrderPercent2 = self.getPercentX(D_A, R_B, int(EachSide), maxZero)
            crossOrderPercent_total = crossOrderPercent1/2 + crossOrderPercent2/2
        else:
            sameOrderPercent_total = self.getPercentExact_same_Position(D_B, R_B, int(EachSide)) + self.getPercentExact_same_Position(D_A, R_A,int(EachSide))
            crossOrderPercent_total = self.getPercentExactPosition(D_B, R_A, int(EachSide)) + self.getPercentExactPosition(D_A, R_B, int(EachSide))

        # Get the matching percent for gene content (compare the genes before the transfers and the genes after the trasfers)
        # same and across comparision then get the max percent
        List_Exact = max(sameOrderPercent_total, crossOrderPercent_total)
        #return list contains the gene families around the transfer event
        return R_All, List_Exact , D_B, D_A, R_B, R_A
            
    # compare 2 lists and find the percent of how much they are the same
    def getPercentX(self, list1, list2, EachSideVal, totalZeros):
        commomGF = []
        CommonItems = 0
        ## count number of '0' to remove the common from EachSideVal
        CountZ1 = list1.count('0')
        CountZ2 = list2.count('0')
        maxZ = max(CountZ1,CountZ2)
        #######################
        for element in list1:
            if element in list2:
                if element != '0':
                    commomGF.append(element)
        CommonItems = len(commomGF)
        ### remove the common zeros and devide by number of items left
        if CommonItems <= 0:
            percentX = 0
        else: 
            if totalZeros <= intGenesNumber:
                EachSideNewVal = (EachSideVal) - maxZ
                percentX = (CommonItems * 100) / float(EachSideNewVal)
            else:
                percentX = (CommonItems * 100) / float(EachSideVal)
        
        return percentX
    
    # compare 2 lists and find the percent of how much they are the same
    def getPercent(self, list1, list2, EachSideVal):
        commomGF = []
        CommonItems = 0
        ## count number of '0' to remove the common from EachSideVal
        CountZ1 = list1.count('0')
        CountZ2 = list2.count('0')
        maxZ = max(CountZ1,CountZ2)
        #######################
        for element in list1:
            if element in list2:
                if element != '0':
                    commomGF.append(element)
        CommonItems = len(commomGF)
        
        if CommonItems <= 0:
            percentX = 0
        else: 
            if maxZ <= intGenesNumber:
                EachSideNewVal = (EachSideVal * 2) - maxZ
                percentX = (CommonItems * 100) / float(EachSideNewVal)
            else:
                percentX = (CommonItems * 100) / float(EachSideVal * 2)
        ### remove the common zeros and devide by number of items left
        '''    
        if minZ < EachSideVal:
            EachSideVal = EachSideVal - minZ
            percentX = (CommonItems * 100) / float(EachSideVal * 2)
        else:
            percentX = 0
        '''
        return percentX
    
            
if __name__ == "__main__":
   main(sys.argv[1:])